/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2021 HiSilicon Limited.
 */

#ifndef __HNSDV_H__
#define __HNSDV_H__

#include <stdio.h>
#include <stdbool.h>

#include <sys/types.h>

#include <infiniband/verbs.h>

#ifdef __cplusplus
extern "C" {
#endif

enum hnsdv_context_attr_flags {
	HNSDV_CONTEXT_FLAGS_DCA = 1 << 0,
};

enum hnsdv_context_comp_mask {
	HNSDV_CONTEXT_MASK_DCA_PRIME_QPS = 1 << 0,
	HNSDV_CONTEXT_MASK_DCA_UNIT_SIZE = 1 << 1,
	HNSDV_CONTEXT_MASK_DCA_MAX_SIZE = 1 << 2,
	HNSDV_CONTEXT_MASK_DCA_MIN_SIZE = 1 << 3,
};

struct hnsdv_context_attr {
	uint64_t flags; /* Use enum hnsdv_context_attr_flags */
	uint64_t comp_mask; /* Use enum hnsdv_context_comp_mask */
	uint32_t dca_prime_qps;
	uint32_t dca_unit_size;
	uint64_t dca_max_size;
	uint64_t dca_min_size;
};

bool hnsdv_is_supported(struct ibv_device *device);
struct ibv_context *hnsdv_open_device(struct ibv_device *device,
				      struct hnsdv_context_attr *attr);

enum hnsdv_qp_create_flags {
	HNSDV_QP_CREATE_ENABLE_DCA_MODE = 1 << 0,
	HNSDV_QP_CREATE_ENABLE_UD_SL = 1 << 1,
};

enum hnsdv_qp_congest_ctrl_type {
	HNSDV_QP_CREATE_ENABLE_DCQCN = 1 << 0,
	HNSDV_QP_CREATE_ENABLE_LDCP = 1 << 1,
	HNSDV_QP_CREATE_ENABLE_HC3 = 1 << 2,
	HNSDV_QP_CREATE_ENABLE_DIP = 1 << 3,
};

enum hnsdv_qp_init_attr_mask {
	HNSDV_QP_INIT_ATTR_MASK_QP_CREATE_FLAGS	= 1 << 0,
	HNSDV_QP_INIT_ATTR_MASK_QP_CONGEST_TYPE	= 1 << 1,
};

struct hnsdv_qp_init_attr {
	uint64_t comp_mask;	/* Use enum hnsdv_qp_init_attr_mask */
	uint32_t create_flags;	/* Use enum hnsdv_qp_create_flags */
	uint8_t congest_type; /* Use enum hnsdv_qp_congest_ctrl_type */
};

struct ibv_qp *hnsdv_create_qp(struct ibv_context *context,
			       struct ibv_qp_init_attr_ex *qp_attr,
			       struct hnsdv_qp_init_attr *hns_qp_attr);

enum hnsdv_query_context_comp_mask {
	HNSDV_CONTEXT_MASK_CONGEST_TYPE = 1 << 0,
};

struct hnsdv_context {
	uint64_t comp_mask; /* use enum hnsdv_query_context_comp_mask */
	uint64_t flags;
	uint8_t congest_type; /* Use enum hnsdv_qp_congest_ctrl_type */
};

int hnsdv_query_device(struct ibv_context *ctx_in,
		       struct hnsdv_context *attrs_out);

#ifdef __cplusplus
}
#endif

#endif /* __HNSDV_H__ */
